<?php

namespace Drupal\social_auth\Plugin\Network;

use Drupal\Core\Url;
use Drupal\social_api\Plugin\NetworkBase as SocialApiNetworkBase;
use Drupal\social_api\SocialApiException;
use Drupal\social_auth\Settings\SettingsBase;

/**
 * Defines a Network Plugin for Social Auth.
 */
abstract class NetworkBase extends SocialApiNetworkBase implements NetworkInterface {

  /**
   * {@inheritdoc}
   */
  protected function initSdk(): mixed {
    $network = $this->networkManager->getDefinition($this->pluginId);
    if (!class_exists($network['class_name'])) {
      throw new SocialApiException("Library class not found: {$network['class_name']}");
    }

    if ($this->validateConfig($this->settings)) {
      // All these settings are mandatory.
      $league_settings = [
        'clientId' => $this->settings->getClientId(),
        'clientSecret' => $this->settings->getClientSecret(),
        'redirectUri' => $this->getCallbackUrl()->setAbsolute()->toString(),
      ];

      // Proxy configuration data for outward proxy.
      $proxyUrl = $this->siteSettings->get('http_client_config')['proxy']['http'] ?? NULL;
      if ($proxyUrl) {
        $league_settings['proxy'] = $proxyUrl;
      }
      return new $network['class_name']($league_settings);
    }

    throw new SocialApiException('Module configuration validation failed -- verify client ID and secret settings.');
  }

  /**
   * Checks that module is configured.
   *
   * @param \Drupal\social_auth\Settings\SettingsBase $settings
   *   Social Auth implementor settings.
   *
   * @return bool
   *   True if module is configured. False otherwise.
   */
  protected function validateConfig(SettingsBase $settings): bool {
    if (!$settings->getClientId() || !$settings->getClientSecret()) {
      $this->loggerFactory
        ->get($this->pluginId)
        ->error('Define Client ID and Client Secret in module settings.');
      return FALSE;
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirectUrl(array $route_options = []): Url {
    return $this->getUrlFromDefaultRoute('redirect', $route_options);
  }

  /**
   * {@inheritdoc}
   */
  public function getCallbackUrl(array $route_options = []): Url {
    return $this->getUrlFromDefaultRoute('callback', $route_options);
  }

  /**
   * {@inheritdoc}
   */
  public function getSettingsFormUrl(array $route_options = []): Url {
    return $this->getUrlFromDefaultRoute('settings_form', $route_options);
  }

  /**
   * {@inheritdoc}
   */
  public function getProviderLogoPath(): string {
    $network = $this->networkManager->getDefinition($this->pluginId);
    $module_path = $this->networkManager->getModuleHandler()->getModule($network['provider'])->getPath();
    return base_path() . "$module_path/{$network['img_path']}";
  }

  /**
   * Gets a URL using the default routes provided by Social Auth.
   *
   * @param string $type
   *   Route type.
   * @param array $route_options
   *   Route options.
   *
   * @return \Drupal\Core\Url
   *   URL object for the requested route type.
   */
  private function getUrlFromDefaultRoute(string $type, array $route_options = []): Url {
    return Url::fromRoute(
      $this->getPluginDefinition()['routes'][$type],
      ['network' => $this->getPluginDefinition()['short_name']],
      $route_options
    );
  }

}
